@@ -1,5 +1,5 @@
 /***************************************************************************************************
- * Copyright 2023 NVIDIA Corporation. All rights reserved.
+ * Copyright 2025 NVIDIA Corporation. All rights reserved.
  **************************************************************************************************/
 /// \file
 /// \brief Database transactions.
@@ -9,6 +9,7 @@
 
 #include <mi/base/interface_declare.h>
 #include <mi/neuraylib/type_traits.h>
+#include <mi/neuraylib/version.h> // for MI_NEURAYLIB_DEPRECATED_ENUM_VALUE
 
 namespace mi {
 
@@ -35,15 +36,14 @@ class IScope;
 /// then you have to serialize all transaction uses. This does not only apply to methods of
 /// #mi::neuraylib::ITransaction, but all methods that implicitly use the transaction. For example,
 /// such a use can happen by methods of DB elements returned from #access() or #edit() calls, or by
-/// objects returned from factories taking the transaction as an \ifnot DICE_API argument, like
-/// #mi::neuraylib::IMdl_factory::create_module_builder(). \else argument. \endif
+/// objects returned from factories taking the transaction as an argument, like
+/// #mi::neuraylib::IMdl_factory::create_module_builder().
 ///
 /// \par Concurrent accesses to database elements within a transaction
+///
 /// Access to database elements is provided by #access() (read-only) and #edit() (for modification).
 /// The interface pointers returned by these methods must be released when you are done, in
-/// particular before the transaction is committed or aborted. Releasing the last interface pointer
-/// obtained from #edit() makes the changes also visible to later #edit() calls for the same
-/// database element.
+/// particular before the transaction is committed or aborted.
 /// \par
 /// Note that it is possible to access the same database element concurrently in the same
 /// transaction. Concurrently means that the interface pointer obtained from an earlier #access()
@@ -57,9 +57,9 @@ class IScope;
 ///      the changes as they are done via the interface pointer returned from the last #edit() call.
 ///      \if IRAY_API Note that this use case is not supported for user-defined classes (classes
 ///      derived from #mi::neuraylib::IUser_class).\endif </li>
-/// <li> #edit() call after #access() calls: The changes done via the interface pointer returned from
-///      #edit() are not observable through any interface pointer returned from the #access() calls.
-///      </li>
+/// <li> #edit() call after #access() calls: The changes done via the interface pointer returned
+///      from #edit() are not observable through any interface pointer returned from the #access()
+///      calls.</li>
 /// <li> multiple #edit() calls: The changes done via the individual interface pointers are not
 ///      observable through the other interface pointers, except for the changes done via the
 ///      interface pointer obtained first at the time the second interface pointer is obtained. The
@@ -72,14 +72,46 @@ class IScope;
 /// which internally calls #access().
 ///
 /// \par Concurrent transactions
+///
 /// If the same database element is edited in multiple overlapping transactions, the changes from
 /// the transaction created last survive, independent of the order in which the transactions are
-/// committed. \ifnot MDL_SDK_API If needed, the lifetime of transactions can be serialized across
+/// committed. \if IRAY_API If needed, the lifetime of transactions can be serialized across
 /// hosts (see #mi::neuraylib::IDatabase::lock() for details). \endif
 class ITransaction : public
     mi::base::Interface_declare<0x6ca1f0c2,0xb262,0x4f09,0xa6,0xa5,0x05,0xae,0x14,0x45,0xed,0xfa>
 {
 public:
+    /// Symbolic privacy level for the privacy level of the scope of this transaction.
+    ///
+    /// This symbolic constant can be passed to #store() and #copy() to indicate the privacy level
+    /// of the scope of this transaction. It has the same affect as passing the result of
+    /// #mi::neuraylib::IScope::get_privacy_level(), but is more convenient.
+    static const mi::Uint8 LOCAL_SCOPE = 255;
+
+    /// \name Static properties
+    //@{
+
+    /// Returns the ID of this transaction.
+    ///
+    /// The transaction ID is of most use when debugging an application as the value returned allows
+    /// one to correlate log messages and admin HTTP server output with the API actions.
+    ///
+    /// \return            The ID of the transaction.
+    virtual const char* get_id() const = 0;
+
+    /// Returns the scope of this transaction.
+    virtual IScope* get_scope() const = 0;
+
+    //@}
+    /// \name Committing and aborting
+    //@{
+
+    /// Indicates whether the transaction is open.
+    ///
+    /// \return   \c true if the transaction is still open, or \c false if the transaction is
+    ///           closed, i.e., it has been committed or aborted.
+    virtual bool is_open() const = 0;
+
     /// Commits the transaction.
     ///
     /// Note that a commit() implicitly closes the transaction.
@@ -97,11 +129,9 @@ public:
     /// A closed transaction does not allow any future operations and needs to be released.
     virtual void abort() = 0;
 
-    /// Indicates whether the transaction is open.
-    ///
-    /// \return   \c true if the transaction is still open, or \c false if the transaction is
-    ///           closed, i.e., it has been committed or aborted.
-    virtual bool is_open() const = 0;
+    //@}
+    /// \name Creation of DB elements
+    //@{
 
     /// Creates an object of the type \p type_name.
     ///
@@ -133,12 +163,12 @@ public:
     ///                     of the object to create.
     /// \param argv         The array of arguments passed to the constructor. Passed to the
     ///                     constructor of factory of the object to create.
-    /// \return             A pointer to the created object, or \c NULL if \p type_name is invalid
-    ///                     (\c NULL pointer) or not a valid type name.
+    /// \return             A pointer to the created object, or \c nullptr if \p type_name is
+    ///                     invalid (\c nullptr) or not a valid type name.
     virtual base::IInterface* create(
         const char* type_name,
         Uint32 argc = 0,
-        const base::IInterface* argv[] = 0) = 0;
+        const base::IInterface* argv[] = nullptr) = 0;
 
     /// Creates an object of the type \p type_name.
     ///
@@ -187,14 +217,14 @@ public:
     /// \param argv         The array of arguments passed to the constructor. Passed to the
     ///                     constructor of factory of the object to create.
     /// \tparam T           The interface type of the class to create.
-    /// \return             A pointer to the created object, or \c NULL if \p type_name is invalid
-    ///                     (\c NULL pointer), not a valid type name, or does not create an object
-    ///                     of type \c T.
+    /// \return             A pointer to the created object, or \c nullptr if \p type_name is
+    ///                     invalid (\c nullptr), not a valid type name, or does not create an
+    ///                     object of type \c T.
     template<class T>
     T* create(
         const char* type_name,
         Uint32 argc = 0,
-        const base::IInterface* argv[] = 0)
+        const base::IInterface* argv[] = nullptr)
     {
         base::IInterface* ptr_iinterface = create( type_name, argc, argv);
         if( !ptr_iinterface)
@@ -244,12 +274,10 @@ public:
         return create<T>( Type_traits<T>::get_type_name());
     }
 
-    /// Symbolic privacy level for the privacy level of the scope of this transaction.
-    ///
-    /// This symbolic constant can be passed to #store() and #copy() to indicate the privacy level
-    /// of the scope of this transaction. It has the same affect as passing the result of
-    /// #mi::neuraylib::IScope::get_privacy_level(), but is more convenient.
-    static const mi::Uint8 LOCAL_SCOPE = 255;
+    //@}
+
+    /// \name Storing of DB elements
+    //@{
 
     /// Stores the element \p db_element in the database under the name \p name and with the privacy
     /// level \p privacy.
@@ -269,8 +297,8 @@ public:
     ///
     /// \param db_element The #mi::base::IInterface to store.
     /// \param name       The name under which to store \p db_element. If there exists already a DB
-    ///                   element with that name then it will be overwritten \if IRAY_API (but see
-    ///                   also return code -9 below) \endif
+    ///                   element with that name then it will be overwritten (but see also return
+    ///                   code -9 below),
     /// \param privacy    The privacy level under which to store \p db_element (in the range from 0
     ///                   to the privacy level of the scope of this transaction). In addition, the
     ///                   constant #LOCAL_SCOPE can be used as a shortcut to indicate the privacy
@@ -279,7 +307,7 @@ public:
     /// \return
     ///        -  0: Success.
     ///        - -1: Unspecified failure.
-    ///        - -2: Invalid parameters (\c NULL pointer).
+    ///        - -2: Invalid parameters (\c nullptr).
     ///        - -3: The transaction is not open.
     ///        - -4: \p db_element is not a DB element.
     ///        - -5: Invalid privacy level.
@@ -298,14 +326,18 @@ public:
     virtual Sint32 store(
         base::IInterface* db_element, const char* name, Uint8 privacy = LOCAL_SCOPE) = 0;
 
+    //@}
+    /// \name Accessing and editing DB elements
+    //@{
+
     /// Retrieves an element from the database.
     ///
     /// The database searches for the most recent version of the named DB element visible for the
     /// current transaction. That version will be returned.
     ///
     /// \param name   The name of the element to retrieve.
-    /// \return       The requested element from the database, or \c NULL if \p name is invalid, no
-    ///               DB element with that name exists, or the transaction is already closed.
+    /// \return       The requested element from the database, or \c nullptr if \p name is invalid,
+    ///               no DB element with that name exists, or the transaction is already closed.
     virtual const base::IInterface* access( const char* name) = 0;
 
     /// Retrieves an element from the database.
@@ -321,15 +353,15 @@ public:
     ///
     /// \param name   The name of the element to retrieve.
     /// \tparam T     The interface type of the element to retrieve.
-    /// \return       The requested element from the database, or \c NULL if \p name is invalid, no
-    ///               DB element with that name exists, the transaction is already closed, or the
-    ///               element is not of type \c T.
+    /// \return       The requested element from the database, or \c nullptr if \p name is invalid,
+    ///               no DB element with that name exists, the transaction is already closed, or
+    ///               the element is not of type \c T.
     template<class T>
     const T* access( const char* name)
     {
         const base::IInterface* ptr_iinterface = access( name);
         if( !ptr_iinterface)
-            return 0;
+            return nullptr;
         const T* ptr_T = static_cast<const T*>( ptr_iinterface->get_interface( typename T::IID()));
         ptr_iinterface->release();
         return ptr_T;
@@ -344,8 +376,8 @@ public:
     /// manually in the database using the #store() method.
     ///
     /// \param name   The name of the element to retrieve.
-    /// \return       The requested element from the database, or \c NULL if \p name is invalid, no
-    ///               DB element with that name exists, or the transaction is already closed.
+    /// \return       The requested element from the database, or \c nullptr if \p name is invalid,
+    ///               no DB element with that name exists, or the transaction is already closed.
     virtual base::IInterface* edit( const char* name) = 0;
 
     /// Retrieves an element from the database and returns it ready for editing.
@@ -364,20 +396,23 @@ public:
     ///
     /// \param name   The name of the element to retrieve.
     /// \tparam T     The interface type of the element to retrieve.
-    /// \return       The requested element from the database, or \c NULL if \p name is invalid, no
-    ///               DB element with that name exists, the transaction is already closed, or the
-    ///               element is not of type \c T.
+    ///               no DB element with that name exists, the transaction is already closed, or
+    ///               the element is not of type \c T.
     template<class T>
     T* edit( const char* name)
     {
         base::IInterface* ptr_iinterface = edit( name);
         if( !ptr_iinterface)
-            return 0;
+            return nullptr;
         T* ptr_T = static_cast<T*>( ptr_iinterface->get_interface( typename T::IID()));
         ptr_iinterface->release();
         return ptr_T;
     }
 
+    //@}
+    /// \name Copies, removal, and properties of DB elements
+    //@{
+
     /// Creates a copy of a database element.
     ///
     /// Note that DB elements of type #mi::neuraylib::IModule and
@@ -392,7 +427,7 @@ public:
     ///                  itself.
     /// \return
     ///                  -  0: Success.
-    ///                  - -2: Invalid parameters (\c NULL pointer).
+    ///                  - -2: Invalid parameters (\c nullptr).
     ///                  - -3: The transaction is not open.
     ///                  - -4: There is no DB element named \p source visible in this transaction.
     ///                  - -5: Invalid privacy level.
@@ -410,10 +445,14 @@ public:
 
     /// Marks the element with the name \p name for removal from the database.
     ///
-    /// Note that the element continues to be stored in the database as long as it is referenced by
-    /// other elements. If it is no longer referenced, and the last transaction were it was
-    /// referenced has been committed, it will be lazily removed by the garbage collection of the
-    /// DB. There is no guarantee when this will happen.
+    /// \par Global removals
+    ///
+    /// The purpose of global removals is to mark all versions of a database element for garbage
+    /// collection. Such a marker has no effect while the element is still referenced (in any scope)
+    /// by other elements or while the transaction where the removal request was made is still open.
+    /// When these conditions do no longer apply, the element becomes eligible for garbage
+    /// collection and must no longer be used in any way. There is no guarantee when the garbage
+    /// collection will actually remove the element.
     ///
     /// This implies that a #remove() call might actually remove an element that was stored later
     /// under the same name. This can potentially lead to invalid tag accesses. Those cases can be
@@ -421,32 +460,57 @@ public:
     /// committed and before starting the next one to force garbage collection of all possible
     /// elements.
     ///
-    /// \if IRAY_API
     /// See also \ref mi_neuray_database_reuse_of_names for more details and correct usage patterns.
-    /// \endif
     ///
-    /// \param name           The name of the element in the database to mark for removal.
-    /// \param only_localized \if MDL_SDK_API Unused. \else If \c true, the element is only removed
-    ///                       if it exists in the scope of the transaction; parent scopes are not
-    ///                       considered. \endif
-    /// \return
-    ///                       -  0: Success.
-    ///                       - -1: There is no DB element named \p name visible in this
-    ///                             transaction (\p only_localize is \c false) or there is no
-    ///                             DB element named \p name in the scope of this transaction
-    ///                             (\p only_localized is \c true).
-    ///                       - -2: Invalid parameters (\c NULL pointer).
-    ///                       - -3: The transaction is not open.
+    /// \par Local removals
+    ///
+    /// The purpose of local removals is to undo the effects of an earlier localization via
+    /// #mi::neuraylib::ITransaction::copy(). A local removal request requires that the element
+    /// exists in the scope of the transaction, and at least one more version of that element
+    /// exists in one of the parent scopes. The effect of a local removal request is to immediately
+    /// hide the version the scope of the transaction (the \em local copy), and to make the next
+    /// version in one of the parent scopes accessible from the very same transaction. The hidden
+    /// local copy will be lazily removed by the garbage collection of the DB. There is no
+    /// guarantee when this will happen.
+    ///
+    /// \param name                The name of the element in the database to mark for removal.
+    /// \param only_localized      \c false for global removals (the default) or \c true for local
+    ///                            removals. The flag is ignored in favor of global removals if the
+    ///                            transaction belongs to the global scope.
+    /// \return                    -  0: Success (including subsequent global removals on elements
+    ///                                  already marked for global removal).
+    ///                            - -1: For global removals: there is no DB element named \p name
+    ///                                  visible in this transaction. For local removals: there is
+    ///                                  no DB element named \p name in the scope of this
+    ///                                  transaction or there is no version of that DB element in
+    ///                                  one of the parent scopes.
+    ///                            - -2: Invalid parameters (\c nullptr).
+    ///                            - -3: The transaction is not open.
     virtual Sint32 remove( const char* name, bool only_localized = false) = 0;
 
     /// Returns the name of a database element.
     ///
     /// \param db_element   The DB element.
-    /// \return             The name of the DB element, or \c NULL if \p db_element is invalid
-    ///                     (\c NULL pointer), the object is not in the database, or the
+    /// \return             The name of the DB element, or \c nullptr if \p db_element is invalid
+    ///                     (\c nullptr), the object is not in the database, or the
     ///                     transaction is already closed.
     virtual const char* name_of( const base::IInterface* db_element) const = 0;
 
+    /// Returns the privacy level of the element with the name \p name.
+    ///
+    /// \param name          The name of the element.
+    /// \return
+    ///                      - >= 0: Success. The privacy level of the element (in the range 0-255).
+    ///                      -   -2: Invalid parameters (\c nullptr).
+    ///                      -   -3: The transaction is not open.
+    ///                      -   -4: There is no DB element named \p name visible in this
+    ///                              transaction.
+    virtual Sint32 get_privacy_level( const char* name) const = 0;
+
+    //@}
+    /// \name Time stamps
+    //@{
+
     /// Returns the time stamp describing the current "time".
     ///
     /// \note The time stamp is not related to time in the classical meaning. It rather relates to
@@ -502,16 +566,9 @@ public:
     virtual bool has_changed_since_time_stamp(
         const char* element, const char* time_stamp) const = 0;
 
-    /// Returns the ID of this transaction.
-    ///
-    /// The transaction ID is of most use when debugging an application as the value returned allows
-    /// one to correlate log messages and admin HTTP server output with the API actions.
-    ///
-    /// \return            The ID of the transaction.
-    virtual const char* get_id() const = 0;
-
-    /// Returns the scope of this transaction.
-    virtual IScope* get_scope() const = 0;
+    //@}
+    /// \name Traversal
+    //@{
 
     /// Returns scene elements of a subgraph originating at a given scene element.
     ///
@@ -530,30 +587,21 @@ public:
     /// \param name_pattern   An extended regular expression that acts as filter on the names of
     ///                       returned scene elements (see [\ref OGBS7]). The regular expression
     ///                       is matched to \em any \em part of the scene element name, not just to
-    ///                       the \em entire scene element name. The value \c NULL is handled as
+    ///                       the \em entire scene element name. The value \c nullptr is handled as
     ///                       \c ".*".
     /// \param type_names     A list of type names that acts as filter on the names of returned
     ///                       scene elements. Only scene elements with a matching type name pass
-    ///                       the filter. The value \c NULL lets all scene elements pass the filter
-    ///                       irrespective of their type name.
+    ///                       the filter. The value \c nullptr lets all scene elements pass the
+    ///                       filter irrespective of their type name.
     /// \return               A list of name of scene elements in the subgraph matching the given
-    ///                       regular expression and type name filter, or \c NULL in case of
-    ///                       an invalid root element name or an invalid regular expression.
+    ///                       regular expression and type name filter, or \c nullptr in case of an
+    ///                       invalid root element name or an invalid regular expression.
     virtual IArray* list_elements(
         const char* root_element,
-        const char* name_pattern = 0,
-        const IArray* type_names = 0) const = 0;
+        const char* name_pattern = nullptr,
+        const IArray* type_names = nullptr) const = 0;
 
-    /// Returns the privacy level of the element with the name \p name.
-    ///
-    /// \param name          The name of the element.
-    /// \return
-    ///                      - >= 0: Success. The privacy level of the element (in the range 0-255).
-    ///                      -   -2: Invalid parameters (\c NULL pointer).
-    ///                      -   -3: The transaction is not open.
-    ///                      -   -4: There is no DB element named \p name visible in this
-    ///                              transaction.
-    virtual Sint32 get_privacy_level( const char* name) const = 0;
+    //@}
 };
 
 /**@}*/ // end group mi_neuray_database_access
